'''

:author: fv
:date: Created on 24 mars 2021
'''
import cmd
import datetime as DT

import dapi2

from dapi2.derror import DApiComError
from dboard.common import DBoardPreferedDapiMode, SystemModeConfig
from dapi2.common import wordToVersion, wordToDate, versionToStr
from dboard.generic import GenericBoard
from dboard.base import DBoard
from PyQt5.QtCore import QCoreApplication




def valToWord(v):
    '''convert a value in 16-bits integer (word)
    
    The value can be expressed as :
    - decimal without prefix
    - hexadecimal with prefix :code:`0x`
    - binary with prefix :code:`0b`
    - octal with prefix :code:`0o`  
    
    :param str v: The value to convert
    :return: An integer
    :rtype: int
    ''' 
    if not isinstance(v, int):
        if v[:1] == '0x':
            v = int(v[2:],16) & 0xFFFF
        elif v[:1] == '0b':
            v = int(v[2:],2) & 0xFFFF
        elif v[:1] == '0o':
            v = int(v[2:],8) & 0xFFFF
        else:
            v = int(v,10) & 0xFFFF
    return v

def valToDate(v):
    '''Convert value to date.
    
    :param str v: The value to convert
    :return: A date
    :rtype: date
    '''
    return DT.datetime.strptime(v, '%Y-%m-%d')

def valToVersion(v):
    '''Convert value to version tuple
    
    :param str v: The value to convert
    :return: An n-tuple containing version numbers
    :rtype: tuple
    '''
    v = v.split('.')
    return tuple((valToWord(x) for x in v))

def parse(arg):
    'Convert a series of zero or more numbers to an argument tuple'
    return tuple(map(int, arg.split()))


class DcpShell(cmd.Cmd):
    prompt = '>'
    file = None
    use_rawinput = False
    
    @classmethod
    def tr(cls, text, disambiguation=None, n=-1):
        return QCoreApplication.translate('DcpShell',text, disambiguation, n)    

    def __init__(self, app, completekey='tab', stdin=None, stdout=None):
        super().__init__(completekey, stdin, stdout)
        
        self.app = app
        
    def onecmd(self, line):
        try:
            return super().onecmd(line)
        except DApiComError as e:
            print(self.tr("*ERROR 0x{0:02X} ({0:d}): {1:s}").format(e.errorno[0], e.label))
        except (KeyError, ValueError) as e:
            print(self.tr("*ERROR: ")+str(e))
            return False # don't stop



    def _show_cfg(self):
        '''Shows application's configuration.'''
        print("""\033[0mConfiguration:
  Communication: \033[1m{com!s}\033[0m
  DAPI preferred mode: \033[1m{board.dmode.name:s}\033[0m             
            """.format(com=self.app.dcom, dapi=self.app.dapi, board=self.app.board))
    
    def _show_board(self):
        '''Shows board's information and status.'''
        self.app.board.getRegisters('system')
        self.app.board.getRegisters('psvr','wer', refresh=True)
        #self.app.dapi.readRegs(self.app.board.getRegisters('system'))
        print("""\033[0mBoard {board!s}:
  Factory data: SN:\033[1m{snr:05d}\033[0m ; Date:\033[1m{fdr:s}\033[0m ; Hardware:\033[1m{hvr:s}\033[0m
  Firmware: \033[1m{svr:s}\033[0m ; Date:\033[1m{fbdr:s}\033[0m  
  BIOS: \033[1m{bvr:s}\033[0m ; Date:\033[1m{bbdr:s}\033[0m  
        """.format(
                board=self.app.board,
                snr=self.app.board.regs.snr.value,
                fdr=wordToDate(self.app.board.regs.fdr.value).isoformat(),
                hvr=versionToStr(wordToVersion(self.app.board.regs.hvr.value)),
                svr=versionToStr(self.app.board.getFirmwareVersion()),
                fbdr=self.app.board.getFirmwareDate().isoformat(),
                bvr=versionToStr(wordToVersion(self.app.board.regs.bvr.value)),
                bbdr=wordToDate(self.app.board.regs.bbdr.value).isoformat(), 
                )
        )
        
        firm = self.app.firmwares.get(
                        self.app.board,
                        self.app.board.getFirmwareTag()
                        #self.app.board.getFirmwareVersion(),
                        #self.app.board.getFirmwareDate()
                        )
        if firm is not None:
            print("  Firmware : " + str(firm))
        else:
            print("  Firmware : undefined (0x{0:04x})".format(self.app.board.getFirmwareTag()) )
            
        if self.app.board.getError():
            print("  Error status: \033[1m0x{0:04X}\033[0m".format(self.app.board.getError()))
        print("  Reset type: "+self.app.board.getLastReset().name )
        print("  Connection access level: "+self.app.board.getAccessLevel().name )
        print("  Power supply: \033[1m{0:.02f}\033[0mV".format(self.app.board.getPowerSupply()))
        if self.app.board.isOnStandby(): 
            print("  On standby")
        else:
            print("  Workspace : {0!s}".format( self.app.board.getWorkspace()))
        if len(self.app.board.workspaces) == 1:
            print("  \033[31mWARNING: No workspace found!\033[0m")
            
    def _show_ws(self):
        '''Shows workspaces's information and status.'''
        pass

    def _show_motor(self):
        '''Shows motor's status.'''
        if self.app.board.isOnStandby():
            print('Standby.')
        else:
            self.app.board.getRegisters('scr','acr','smr','msr','a256dcr','a256dvr', 'lsr','rsr', refresh=True)
            mode = self.app.board.getSystemModeConfiguration()
            print("""\033[0mMotor:
  Set points: speed = \033[1m{r.scr.value:d}\033[0mrpm ; acceleration = \033[1m{r.acr.value:d}\033[0mkrpm/s
  Mode: \033[1m0x{r.smr.value:04x}\033[0m = {m!s} 
  Requested speed = \033[1m{r.rsr.value:d}\033[0mrpm ; Logical speed = \033[1m{r.lsr.value:d}\033[0mrpm
  Real speed = \033[1m{r.msr.value:d}\033[0mrpm ; current = \033[1m{r.a256dcr.value:d}\033[0mmA ; voltage = \033[1m{r.a256dvr.value:d}\033[0mmV
""".format(r=self.app.dapi.regs, m=mode))
            
            
    
    def do_cfg(self, arg):
        '''Modify or show the application configuration.
        Usage: cfg [dmode {c,r}] [show]
        dmode c : set the preferred DAPI mode to `command`. 
        dmode r : set the preferred DAPI mode to `register`.
        show : display the appication configuration
        '''
        args = arg.lower().split()
        
        if args[0] == 'dmode':
            if args[1] == 'c':
                self.app.board.dmode = DBoardPreferedDapiMode.COMMAND
            elif args[1] == 'r':
                self.app.board.dmode = DBoardPreferedDapiMode.REGISTER
            else:
                raise ValueError('Invalid DAPI2 mode!')
        elif args[0] == 'show':
            self._show_cfg()
    
    def do_sb(self, arg):
        '''Shows information about the board.
        Usage: sb'''
        self._show_board()

    def do_sm(self, arg):
        '''Shows information about the motor.
        Usage: sm'''
        self._show_motor()
    
    def do_sc(self, arg):
        '''Shows information about the application configuration.
        Usage: sc'''
        self._show_cfg()
    
    def do_show(self, arg):
        '''Shows information about an item.
        Usage: show {cfg, board, ws, motor}
        '''
        args = arg.lower().split()
        if args[0] == 'cfg':
            self._show_cfg()
        elif args[0] == 'board':
            self._show_board()
        elif args[0] == 'ws':
            self._show_ws()
        elif args[0] == 'motor':
            self._show_motor()
        else:
            raise ValueError('Invalid item to show!')
        
    def do_connect(self, arg):
        '''Connect to board.
        Usage: connect {no, user, service, factory}
        '''
        level = dapi2.DApiAccessLevel[arg.upper()]
        self.app.board.connect(level, level.passwd )
        self.prompt = level.name[0]+'>'
        
    def do_cu(self, arg):
        '''Connect to board with `user` level.
        Usage: cu
        '''
        self.do_connect('user')

    def do_cs(self, arg):
        '''Connect to board with `service` level.
        Usage: cs
        '''
        self.do_connect('service')

    def do_cf(self, arg):
        '''Connect to board with `factory` level.
        Usage: cf
        '''
        self.do_connect('factory')

    def do_disconnect(self, arg):
        '''Disconnect board.
        Usage: disconnect'''
        self.app.board.disconnect()
        self.prompt = '>'
        
    def do_start(self, arg):
        '''Starts the motor.
        Usage: start [<speed>]
        
        <speed> : optional argument to fix new speed set point.'''
        args = parse(arg)
        try:
            speed = args[0]
        except:
            speed = None
        self.app.board.motorStart(speed)
        
    def do_mode(self, arg):
        '''Change system mode 
        Usage: mode {set,clear} <flag> [<flag> ...]
        
        <flag> : is the configuration flag : {START, HOLDING, ROCKING, AUTOBOOST, AUTOSTOP, AUTOREVERSE, REVERSE, QUADRATIC, LIGHT, FREEWHEEL, INDIRECT, LIGHTAUTO}        
        '''
        
        flags = self.app.board.getSystemModeConfiguration()
        print('Actual system mode: \033[1m0x{f.value:04x}\033[0m = {f!s} '.format(f=flags) )
        to_set = True
        
        args = list(a.upper() for a in arg.split())
        if len(args) > 0:
            try:
                for arg in args:
                    if arg == 'SET':
                        to_set = True 
                    elif arg == 'CLEAR':
                        to_set = False
                    else:
                        if to_set:
                            flags |= SystemModeConfig[arg]
                        else:
                            flags &= ~SystemModeConfig[arg]
                self.app.board.setSystemModeConfiguration(flags)
                print('New system mode: \033[1m0x{f.value:04x}\033[0m = {f!s} '.format(f=flags) )
            except Exception:
                print("""
Malformed command line. Usage: mode {set,clear} <flag> [<flag> ...]
Example: mode set indirect clear light""")
                return False                    
        
    def do_stop(self, arg):
        '''Stops the motor.
        Usage: stop'''
        self.app.board.motorStop()
        
    def do_light(self, arg):
        '''Sets light
        Usage: light <cmd>
        
        <cmd> : is light command.   
        '''
        args = (a.lower() for a in arg.split())
        for a in args:
            if a == 'on':
                self.app.board.lightOn() 
            elif a == 'off':
                self.app.board.lightOff()
            if a == 'auto':
                self.app.board.lightAuto(True)
            elif a == 'manu':
                self.app.board.lightAuto(False)
            if a.isdigit():
                self.app.board.lightIntensity(int(a))
    
    def do_ws0(self, arg):    
        '''Go to standby mode
        Usage: ws0
        ''' 
        self.app.board.setWorkspace(0)

    def do_ws1(self, arg):    
        '''Go to workspace #1
        Usage: ws1
        ''' 
        self.app.board.setWorkspace(1)

    def do_ws2(self, arg):    
        '''Go to workspace #2
        Usage: ws2
        ''' 
        self.app.board.setWorkspace(2)
    
    def do_ws(self, arg):
        '''Change the active workspace
        Usage: ws <workspace>
        
        <workspace> is the Workspace number (=PAR)
        ''' 
        self.app.board.setWorkspace(int(arg))
        
    def do_get(self, arg):
        '''Gets the value of one or more registers.
        Usage: get <reg> [;<reg> ...]
        
        <reg> is the register name
        '''
        args = (a.lower() for a in arg.split())
        regs = self.app.board.getRegisters(*args)
        print("=> "+' ; '.join( r.toString() for r in regs) )
        #print("=> "+' ; '.join( "{0:s}=0x{1:04x}({1:d})".format(r.name, r.value) for r in regs) )
        
    def do_set(self, arg):
        '''Sets the value of one or more registers.
        Usage: set <reg>=<val> [;<reg>=<val> ...]
        
        <reg> is the register name
        <val> is the new value to set
        ''' 
        arg = arg.replace(' ', '').lower()
        try:
            args = dict([ (k,v) for k,v in [ tuple(a.split('=')) for a in arg.split(';') ]])
            
            for k, v in args.items():
                if isinstance(v, str):
                    if v[:2] == '0x':
                        args[k] = int(v[2:],16) & 0xFFFF
                    elif v[:2] == '0b':
                        args[k] = int(v[2:],2) & 0xFFFF
                    else:
                        args[k] = int(v,10) & 0xFFFF
            
        except Exception:
            print("""
Malformed command line. Usage: set reg <reg> = <value> [; <reg> = <value>  ... ]
Where:
  <reg> is the register name
  <value> is the register the value expressed in decimal (12), hexadecimal (0xC) or binary (0b1100).""")
            return False
        self.app.board.setRegisters(**args, synchronous=True)
        # r = []
        # for k,v in args.items():
            # r.append("{0:s} = 0x{1:04x} ({1:d})".format(k, self.app.board.regs(k).value ))
        # print("  "+" ; ".join(r))
        regs = self.app.board.getRegisters(*args.keys())
        print("=> "+' ; '.join( "{0:s}=0x{1:04x}({1:d})".format(r.name, r.value) for r in regs) )
        
        
    def _flashCallback(self, i, n):
        p = i/n
        print('\033[2K\r{0!s} [{1:50s}] {2:6.2f}% ({3:d}/{4:d})'.format('Writing in progress', '#'*int(round(50*p)), 100*p, i, n), end='')
        
    def do_firm(self, arg):
        '''Rewrites the firmware into the board ROM.
        Usage: firm
        '''
        
        if isinstance(self.app.board, GenericBoard):  
            print("Choice the board family target:")
            board_classes = list(DBoard.getBoardClasses().values())
            for i, board_class in enumerate(board_classes):
                print("{0:3d} - {1:s}".format(i+1, board_class.getName()))
            print("Enter 1 to {0:d} to choice board family or 0 to abort. [0]".format(i))
            s = input()
            if s == '' or s == '0':
                return False    
            board_class = board_classes[int(s,10)-1]    
        else:
            board_class = self.app.board.__class__
        
        print("Choice the firmware to write into {0:s} ROM:".format(board_class.getName()))
        firmwares = list(self.app.firmwares.find(board_class))
        for i, firm in enumerate(firmwares):
            print("{0:4d} - {1!s} ".format(i+1, firm)) 
        print("Enter 1 to {0:d} to choice the firmware or 0 to abort. [0]".format(i+1))
        s = input()
        if s == '' or s == '0':
            return False
        i = int(s,10)
        firm = firmwares[i-1]
        print("Are your sure to overwrite the {0:s} board firmware with this {1!s} (Yes / No)? y[n]".format(board_class.getName(), firm))
        s = input()
        if s.lower() == 'y' :
            self.app.tracer.stop = True
            try:
                self.app.board.flashFirm(firm, self._flashCallback)
            finally:
                self.app.tracer.stop = False
            print('\033[2K\rFirmware update operation completed successfully.')
        else:
            return False
        
    def do_factory(self, arg):
        '''Rewrites factory data into the board E²PROM.
        Usage: factory [sn=<sn>][;fd=<fd>][;hv=<hv>]
        
        Where:
          <sn> is the serial number
          <fd> is the date in ISO format (YYYY-MM-DD)
          <hv> is the major and minor hardware version separated by a dot (.)      
        '''
        
        data = list(self.app.board.getFactoryData())
        arg = arg.replace(' ', '').lower()
        print('Actual factory data: SN:\033[1m{sn:05d}\033[0m ; Date:\033[1m{fd:s}\033[0m ; Hardware:\033[1m{hv[0]:d}.{hv[1]:02d}\033[0m'.format(sn=data[0], fd=data[1].isoformat(), hv=data[2] ) )
        if len(arg) > 0:  
            try:
                args = dict([ (k,v) for k,v in [ tuple(a.split('=')) for a in arg.split(';') ]])
                for k, v in args.items():
                    if k == 'sn':
                        data[0] = valToWord(v)
                    elif k == 'fd':
                        data[1] = valToDate(v)
                    elif k == 'hv':
                        data[2] = valToVersion(v)
                self.app.board.setFactoryData(*data)
            except Exception:
                print("""
    Malformed command line. Usage: factory [sn=<sn>][;fd=<fd>][;hv=<hv>]
    Example: factory sn=01234 ; fd=2021-05-1 ; hv=5.01""")
                return False
        
        
        
    def do_bye(self, arg):
        '''Terminates the shell'''
        return self.do_exit(arg)

    def do_exit(self, arg):
        '''Terminates the shell'''
        self.close()
        return True
    
    def close(self):
        if self.file:
            self.file.close()
            self.file = None    


from setuptools import setup, setuptools
import sys, os
sys.path.insert(0, os.path.abspath('./src/dsmcp/app'))
from _version import __version__ as APP_VERSION
from common import APP_NAME

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
  
setup(
    name=APP_NAME,
    version=APP_VERSION,
    author="Fabrice Voillat",
    author_email="dev@dassym.com",
    description="The PyDapi2 library offers a Python implementation of the Dassym API version 2.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires = ['lxml', 'pyserial', 'PyDapi2', 'chardet'],
    url="https://github.com/dassym/dsmcp",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.8",
    include_package_data=True
)


'''

:author: fv
:date: Created on 5 mai 2021
'''
from PyQt5.Qt import QDialog, QIcon

from .ui_sendfirmwaredialog import Ui_SendFirmwareDialog
import logging
from app.base import BaseApp

class SendFirmwareDialog(QDialog, Ui_SendFirmwareDialog):
    
    def __init__(self, parent):
        QDialog.__init__(self, parent)
        self.app = parent.app
        self.log = logging.getLogger(self.__class__.__name__)
        self.log.debug( 'Initialize' )
        self.setupUi(self)
        
        self.icons = {'O': QIcon(':/img/ok.svg'),
                    'W': QIcon(':/img/warning.svg'),
                    'E': QIcon(':/img/error.svg')
                    }
        
        firm = self.app.firmwares.get(
                        self.app.board,
                        self.app.board.getFirmwareTag(),
                        self.app.board.getFirmwareVersion(),
                        self.app.board.getFirmwareDate(),
                        )
        if firm is not None:
            self.labelActual.setText("Actual: "+str(firm))
        else:
            self.labelActual.setText("Actual: undefined (0x{0:04x})".format(self.app.board.getFirmwareTag()) )
        
        self._updateFirmwareList()
        
        
    def _updateFirmwareList(self):
        self.comboBoxFirmware.clear()
        self.labelFirmwareDesc.clear()
        #self.labelFirmwareIcon.clear()
        self.comboBoxFirmware.addItem(BaseApp.tr('Select firmware...'), None)
        self.comboBoxFirmware.insertSeparator(1)

        for firmware in self.app.firmwares:
            if self.app.firmwares.isLast(firmware):
                icon = self.icons['O']
            else: 
                icon = self.icons['W']
            self.comboBoxFirmware.addItem(icon, str(firmware), firmware)
        
        
        
    def getFirmware(self):
        return self.comboBoxFirmware.currentData()
        
    @property
    def board(self):
        return self.app.board


'''

:author: fv
:date: Created on 24 mars 2021
'''
from .base import BaseCliApp
from .dcpshell import DcpShell
from ..base import BaseApp
from dapi2.dapi2 import DApiAccessLevel


class DevCliApp(BaseCliApp):
    '''Class for *development* CLI application
    
    :param Namespace config: Application configuration
    '''
    NAME = 'dcp-dev'

    def __init__(self, config):
        '''Constructor'''
        super().__init__(config)
        
    def prepareCfg(self, parser):
        super().prepareCfg(parser)
        parser.add_argument("--normal-mode", dest="dev_mode", action='store_false', help=BaseApp.tr("Sets DSMCP execution mode 'development'. (Default=normal mode)"))
        parser.set_defaults(dev_mode=True)
        
        
    def initialize(self):
        super().initialize()
        self.shell = DcpShell(self)
    
    def run(self):
        '''Executes application'''
        self.log.info(BaseApp.tr('Run single shot'))
        level = self.board.getAccessLevel() 
        if level != DApiAccessLevel.NO:
            self.shell.prompt = level.name[0]+">"
        else: 
            self.shell.prompt = ">"
        self.shell.cmdloop(BaseApp.tr('Welcome to \033[1m{0:s}\033[0m version {1:s} shell! Type `help` or `?` to list commands.').format(
                    self.applicationName(),
                    self.applicationVersion()
                    ))
        return self.terminate()
        


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/basicwindow.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_BasicWindow(object):
    def setupUi(self, BasicWindow):
        BasicWindow.setObjectName("BasicWindow")
        BasicWindow.resize(354, 456)
        self.centralwidget = QtWidgets.QWidget(BasicWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.graphicsViewPanel = QSvgPanel(self.centralwidget)
        self.graphicsViewPanel.setObjectName("graphicsViewPanel")
        self.gridLayout.addWidget(self.graphicsViewPanel, 0, 0, 1, 2)
        self.pushButtonStart = QActionButton(self.centralwidget)
        self.pushButtonStart.setObjectName("pushButtonStart")
        self.gridLayout.addWidget(self.pushButtonStart, 2, 0, 1, 1)
        self.pushButtonStop = QActionButton(self.centralwidget)
        self.pushButtonStop.setObjectName("pushButtonStop")
        self.gridLayout.addWidget(self.pushButtonStop, 2, 1, 1, 1)
        self.widgetSpeed = QSpeed(self.centralwidget)
        self.widgetSpeed.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.widgetSpeed.setFrameShadow(QtWidgets.QFrame.Raised)
        self.widgetSpeed.setObjectName("widgetSpeed")
        self.gridLayout.addWidget(self.widgetSpeed, 3, 0, 1, 2)
        self.label = QtWidgets.QLabel(self.centralwidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 4, 0, 1, 2)
        BasicWindow.setCentralWidget(self.centralwidget)
        self.menuBar = QtWidgets.QMenuBar(BasicWindow)
        self.menuBar.setGeometry(QtCore.QRect(0, 0, 354, 22))
        self.menuBar.setObjectName("menuBar")
        self.menuSystem = QtWidgets.QMenu(self.menuBar)
        self.menuSystem.setObjectName("menuSystem")
        self.menuMotor = QtWidgets.QMenu(self.menuBar)
        self.menuMotor.setObjectName("menuMotor")
        self.menuHelp = QtWidgets.QMenu(self.menuBar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuWorkspace = QtWidgets.QMenu(self.menuBar)
        self.menuWorkspace.setObjectName("menuWorkspace")
        self.menuBoard = QtWidgets.QMenu(self.menuBar)
        self.menuBoard.setObjectName("menuBoard")
        BasicWindow.setMenuBar(self.menuBar)
        self.statusBar = QtWidgets.QStatusBar(BasicWindow)
        self.statusBar.setObjectName("statusBar")
        BasicWindow.setStatusBar(self.statusBar)
        self.actionSysConnect = QtWidgets.QAction(BasicWindow)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/img/link.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSysConnect.setIcon(icon)
        self.actionSysConnect.setObjectName("actionSysConnect")
        self.actionMotorStart = QtWidgets.QAction(BasicWindow)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/img/start.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorStart.setIcon(icon1)
        self.actionMotorStart.setObjectName("actionMotorStart")
        self.actionLightEnabled = QtWidgets.QAction(BasicWindow)
        self.actionLightEnabled.setCheckable(True)
        self.actionLightEnabled.setChecked(True)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/img/light.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionLightEnabled.setIcon(icon2)
        self.actionLightEnabled.setObjectName("actionLightEnabled")
        self.actionLightBlue = QtWidgets.QAction(BasicWindow)
        self.actionLightBlue.setCheckable(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/img/light-blue.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionLightBlue.setIcon(icon3)
        self.actionLightBlue.setObjectName("actionLightBlue")
        self.actionMotorCCW = QtWidgets.QAction(BasicWindow)
        self.actionMotorCCW.setCheckable(True)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/img/counterclockwise.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorCCW.setIcon(icon4)
        self.actionMotorCCW.setObjectName("actionMotorCCW")
        self.actionAbout = QtWidgets.QAction(BasicWindow)
        self.actionAbout.setObjectName("actionAbout")
        self.actionQuit = QtWidgets.QAction(BasicWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionAutoRefresh = QtWidgets.QAction(BasicWindow)
        self.actionAutoRefresh.setCheckable(True)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(":/img/autorefresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionAutoRefresh.setIcon(icon5)
        self.actionAutoRefresh.setObjectName("actionAutoRefresh")
        self.actionSysDisconnect = QtWidgets.QAction(BasicWindow)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(":/img/unlink.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionSysDisconnect.setIcon(icon6)
        self.actionSysDisconnect.setObjectName("actionSysDisconnect")
        self.actionRefresh = QtWidgets.QAction(BasicWindow)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(":/img/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionRefresh.setIcon(icon7)
        self.actionRefresh.setObjectName("actionRefresh")
        self.actionHelp = QtWidgets.QAction(BasicWindow)
        icon8 = QtGui.QIcon()
        icon8.addPixmap(QtGui.QPixmap(":/img/help.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionHelp.setIcon(icon8)
        self.actionHelp.setObjectName("actionHelp")
        self.actionMotorCW = QtWidgets.QAction(BasicWindow)
        self.actionMotorCW.setCheckable(True)
        self.actionMotorCW.setChecked(True)
        icon9 = QtGui.QIcon()
        icon9.addPixmap(QtGui.QPixmap(":/img/clockwise.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorCW.setIcon(icon9)
        self.actionMotorCW.setObjectName("actionMotorCW")
        self.actionMotorStop = QtWidgets.QAction(BasicWindow)
        icon10 = QtGui.QIcon()
        icon10.addPixmap(QtGui.QPixmap(":/img/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.On)
        self.actionMotorStop.setIcon(icon10)
        self.actionMotorStop.setObjectName("actionMotorStop")
        self.actionFirmReset = QtWidgets.QAction(BasicWindow)
        self.actionFirmReset.setObjectName("actionFirmReset")
        self.actionAPISendFirmware = QtWidgets.QAction(BasicWindow)
        self.actionAPISendFirmware.setObjectName("actionAPISendFirmware")
        self.actionAboutBoard = QtWidgets.QAction(BasicWindow)
        self.actionAboutBoard.setObjectName("actionAboutBoard")
        self.actionMotorMode = QtWidgets.QAction(BasicWindow)
        self.actionMotorMode.setObjectName("actionMotorMode")
        self.menuSystem.addAction(self.actionRefresh)
        self.menuSystem.addAction(self.actionAutoRefresh)
        self.menuSystem.addSeparator()
        self.menuSystem.addAction(self.actionQuit)
        self.menuMotor.addAction(self.actionMotorMode)
        self.menuMotor.addAction(self.actionMotorStart)
        self.menuMotor.addAction(self.actionMotorStop)
        self.menuMotor.addAction(self.actionLightEnabled)
        self.menuMotor.addAction(self.actionLightBlue)
        self.menuMotor.addAction(self.actionMotorCCW)
        self.menuHelp.addAction(self.actionHelp)
        self.menuHelp.addAction(self.actionAbout)
        self.menuBoard.addAction(self.actionFirmReset)
        self.menuBoard.addAction(self.actionAPISendFirmware)
        self.menuBoard.addAction(self.actionAboutBoard)
        self.menuBar.addAction(self.menuSystem.menuAction())
        self.menuBar.addAction(self.menuBoard.menuAction())
        self.menuBar.addAction(self.menuWorkspace.menuAction())
        self.menuBar.addAction(self.menuMotor.menuAction())
        self.menuBar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(BasicWindow)
        self.actionQuit.triggered.connect(BasicWindow.close)
        QtCore.QMetaObject.connectSlotsByName(BasicWindow)

    def retranslateUi(self, BasicWindow):
        _translate = QtCore.QCoreApplication.translate
        BasicWindow.setWindowTitle(_translate("BasicWindow", "DSMCP"))
        self.pushButtonStart.setText(_translate("BasicWindow", "Start"))
        self.pushButtonStop.setText(_translate("BasicWindow", "Stop"))
        self.label.setText(_translate("BasicWindow", "TextLabel"))
        self.menuSystem.setTitle(_translate("BasicWindow", "System"))
        self.menuMotor.setTitle(_translate("BasicWindow", "Motor"))
        self.menuHelp.setTitle(_translate("BasicWindow", "Help"))
        self.menuWorkspace.setTitle(_translate("BasicWindow", "Workspace"))
        self.menuBoard.setTitle(_translate("BasicWindow", "Board"))
        self.actionSysConnect.setText(_translate("BasicWindow", "Connect"))
        self.actionSysConnect.setToolTip(_translate("BasicWindow", "Connect DCP to board"))
        self.actionMotorStart.setText(_translate("BasicWindow", "Start"))
        self.actionMotorStart.setToolTip(_translate("BasicWindow", "Start the motor"))
        self.actionMotorStart.setShortcut(_translate("BasicWindow", "F2"))
        self.actionLightEnabled.setText(_translate("BasicWindow", "Light Enabled"))
        self.actionLightEnabled.setToolTip(_translate("BasicWindow", "Enable/disable Light"))
        self.actionLightEnabled.setShortcut(_translate("BasicWindow", "F4"))
        self.actionLightBlue.setText(_translate("BasicWindow", "BlueLight"))
        self.actionLightBlue.setToolTip(_translate("BasicWindow", "Switch white / blue light"))
        self.actionLightBlue.setShortcut(_translate("BasicWindow", "F9"))
        self.actionMotorCCW.setText(_translate("BasicWindow", "CCW"))
        self.actionMotorCCW.setToolTip(_translate("BasicWindow", "Set rotation to counter clockwise direction"))
        self.actionMotorCCW.setShortcut(_translate("BasicWindow", "F7"))
        self.actionAbout.setText(_translate("BasicWindow", "About ..."))
        self.actionAbout.setShortcut(_translate("BasicWindow", "Ctrl+F1"))
        self.actionQuit.setText(_translate("BasicWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("BasicWindow", "Ctrl+Q"))
        self.actionAutoRefresh.setText(_translate("BasicWindow", "AutoRefresh"))
        self.actionSysDisconnect.setText(_translate("BasicWindow", "Disconnect"))
        self.actionSysDisconnect.setToolTip(_translate("BasicWindow", "Disconnect DCP from board"))
        self.actionRefresh.setText(_translate("BasicWindow", "Refresh"))
        self.actionRefresh.setToolTip(_translate("BasicWindow", "Refresh"))
        self.actionRefresh.setShortcut(_translate("BasicWindow", "F5"))
        self.actionHelp.setText(_translate("BasicWindow", "Help"))
        self.actionHelp.setShortcut(_translate("BasicWindow", "F1"))
        self.actionMotorCW.setText(_translate("BasicWindow", "CW"))
        self.actionMotorCW.setToolTip(_translate("BasicWindow", "Set rotation to clockwise direction"))
        self.actionMotorCW.setShortcut(_translate("BasicWindow", "F6"))
        self.actionMotorStop.setText(_translate("BasicWindow", "Stop"))
        self.actionMotorStop.setToolTip(_translate("BasicWindow", "Stop the motor"))
        self.actionMotorStop.setShortcut(_translate("BasicWindow", "F3"))
        self.actionFirmReset.setText(_translate("BasicWindow", "Reset"))
        self.actionAPISendFirmware.setText(_translate("BasicWindow", "Send Firmware"))
        self.actionAboutBoard.setText(_translate("BasicWindow", "About board..."))
        self.actionMotorMode.setText(_translate("BasicWindow", "Motor mode..."))
        self.actionMotorMode.setToolTip(_translate("BasicWindow", "Show motor mode dialog"))
        self.actionMotorMode.setShortcut(_translate("BasicWindow", "F10"))
from app.gui.widget.qactionbutton import QActionButton
from app.gui.widget.qspeed import QSpeed
from app.gui.widget.qsvgpanel import QSvgPanel



'''

:author: F. Voillat
:date: Created on 16 mai 2021
'''
from PyQt5.Qt import QSplashScreen

class AppSplashScreen(QSplashScreen):
    
    def __init__(self, app, pixmap, f):
        self.app = app
        QSplashScreen.__init__(self, pixmap, f)
    
    def drawContents(self, painter):
        QSplashScreen.drawContents(self, painter)
        #painter.setPen(Qt.blue)
        painter.font().setPointSize(30)
        painter.drawText(140,125, self.app.applicationName())
        painter.font().setPointSize(20)
        #painter.drawText(240,125, self.app.appname)
        painter.font().setPointSize(10)
        painter.drawText(140,145, "Version {0:s}".format(*self.app.applicationVersion()))
        
        



'''Generic launcher for *dsmcp* applications 


This script is used to launch dsmcp with command

.. code-block:: bash
    python3 -m dsmcp <application> <options>
    
:author: F. Voillat
:date: Created on 12 mai 2021
'''

import sys
from os.path import os


if __name__ == '__main__':
    if sys.version_info < (3,8):
        raise Exception("DSMCP requires at least Python version 3.8")
    
    from dsmcp.app.launcher import Launcher
    
    launcher = Launcher(os.path.dirname(__file__), add_help=True)
    launcher.parser0.add_argument('application',  choices=('basic', 'dev', 'prod', 'service')) 
    launcher.preConfig() 
    
    if launcher.config.no_gui:
        if launcher.config.application == 'dev':
            from dsmcp.app.cli import DevCliApp
            application = launcher.launch(DevCliApp) 
        else:
            from dsmcp.app.cli import BasicCliApp
            application = launcher.launch(BasicCliApp) 
    else:
        if launcher.config.application == 'dev':
            from dsmcp.app.gui import DevGuiApp
            application = launcher.launch(DevGuiApp)
        else:
            from dsmcp.app.gui import BasicGuiApp
            application = launcher.launch(BasicGuiApp)  
    try: 
        application.initialize()
        application.run()
    except Exception as e:
        application.handleError(e)
        sys.exit(1)
    
    



'''

:author:  F. Voillat
:date: 2021-05-10 Creation
:copyright: Dassym SA 2021
'''
from dboard import SystemModeConfig

SystemModeConfigIcon = {
    SystemModeConfig.START : None,
    SystemModeConfig.HOLDING : None,
    SystemModeConfig.ROCKING : ':/img/rocking.svg',
    SystemModeConfig.AUTOBOOST : None,
    SystemModeConfig.AUTOSTOP : ':/img/autostop.svg',
    SystemModeConfig.AUTOREVERSE : ':/img/autoreverse.svg',
    SystemModeConfig.REVERSE : ':/img/reverse.svg',
    SystemModeConfig.QUADRATIC : ':img/quadratic.svg',
    SystemModeConfig.LIGHT : ':/img/light.svg',
    SystemModeConfig.FREEWHEEL : ':/img/freewheel.svg',
    SystemModeConfig.INDIRECT : ':/img/indirect.svg',
    SystemModeConfig.LIGHTAUTO : ':img/lightauto.svg',
    }


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './src/app/gui/dlg/aboutdialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.0
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(403, 225)
        AboutDialog.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(AboutDialog)
        self.frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setLineWidth(0)
        self.frame.setObjectName("frame")
        self.gridLayout = QtWidgets.QGridLayout(self.frame)
        self.gridLayout.setObjectName("gridLayout")
        self.labelApplication = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelApplication.sizePolicy().hasHeightForWidth())
        self.labelApplication.setSizePolicy(sizePolicy)
        self.labelApplication.setAlignment(QtCore.Qt.AlignCenter)
        self.labelApplication.setObjectName("labelApplication")
        self.gridLayout.addWidget(self.labelApplication, 0, 1, 1, 1)
        self.labelVersion = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelVersion.sizePolicy().hasHeightForWidth())
        self.labelVersion.setSizePolicy(sizePolicy)
        self.labelVersion.setAlignment(QtCore.Qt.AlignCenter)
        self.labelVersion.setObjectName("labelVersion")
        self.gridLayout.addWidget(self.labelVersion, 1, 1, 1, 1)
        self.labelLogo = QtWidgets.QLabel(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelLogo.sizePolicy().hasHeightForWidth())
        self.labelLogo.setSizePolicy(sizePolicy)
        self.labelLogo.setMinimumSize(QtCore.QSize(68, 68))
        self.labelLogo.setAlignment(QtCore.Qt.AlignCenter)
        self.labelLogo.setObjectName("labelLogo")
        self.gridLayout.addWidget(self.labelLogo, 0, 0, 2, 1)
        self.verticalLayout.addWidget(self.frame)
        self.labelDescription = QtWidgets.QLabel(AboutDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelDescription.sizePolicy().hasHeightForWidth())
        self.labelDescription.setSizePolicy(sizePolicy)
        self.labelDescription.setAlignment(QtCore.Qt.AlignCenter)
        self.labelDescription.setObjectName("labelDescription")
        self.verticalLayout.addWidget(self.labelDescription)
        self.groupBoxSystem = QtWidgets.QGroupBox(AboutDialog)
        self.groupBoxSystem.setObjectName("groupBoxSystem")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.groupBoxSystem)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.labelSystem = QtWidgets.QLabel(self.groupBoxSystem)
        self.labelSystem.setObjectName("labelSystem")
        self.verticalLayout_2.addWidget(self.labelSystem)
        self.verticalLayout.addWidget(self.groupBoxSystem)
        self.buttonBox = QtWidgets.QDialogButtonBox(AboutDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(AboutDialog)
        self.buttonBox.rejected.connect(AboutDialog.reject)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_translate("AboutDialog", "About DSMCP"))
        self.labelApplication.setText(_translate("AboutDialog", "Application"))
        self.labelVersion.setText(_translate("AboutDialog", "TextLabel"))
        self.labelLogo.setText(_translate("AboutDialog", "LOGO"))
        self.labelDescription.setText(_translate("AboutDialog", "TextLabel"))
        self.groupBoxSystem.setTitle(_translate("AboutDialog", "System"))
        self.labelSystem.setText(_translate("AboutDialog", "TextLabel"))



'''Module for class DialogComPort. This module implement a dialog for choosing a communication port. 

:author: T. Marti
:date: 2021-03-10 Creation
'''
from .ui_selectconnectiondialog import Ui_SelectionConnectionDialog
from PyQt5.QtWidgets import QDialog

class SelectConnectionDialog(QDialog, Ui_SelectionConnectionDialog):
    """class for making a dialog windows for selecting the serial coomunication port
    
    :param list ports: list of available ports
    """
    def __init__(self, options):
        super(SelectConnectionDialog, self).__init__()
        self.setupUi(self)
        self.portIndex = []
        self.portRef = []
        for option in options:
            index = "[{}] {}".format(option[0], option[1])
            self.portIndex.append(index)
            self.portRef.append(option[0])
            self.comboBox.addItem(index)
            
    def getSelectedOption(self):
        """get the selected port of the dialog
        """
        return self.portRef[self.portIndex.index(self.comboBox.currentText())]